@@ -329,10 +329,15 @@ devicepresence_filter (GdkXEvent *xevent,
 static void
 set_devicepresence_handler (GsdMouseManager *manager)
 {
-        Display *display = gdk_x11_get_default_xdisplay ();
+        Display *display;
         XEventClass class_presence;
         int xi_presence;
 
+        if (!supports_xinput_devices ())
+                return;
+
+        display = gdk_x11_get_default_xdisplay ();
+
         gdk_error_trap_push ();
         DevicePresence (display, xi_presence, class_presence);
         XSelectExtensionEvent (display,
@@ -624,7 +629,7 @@ set_mousetweaks_daemon (GsdMouseManager *manager,
                         gtk_window_set_title (GTK_WINDOW (dialog),
                                               _("Mouse Preferences"));
                         gtk_window_set_icon_name (GTK_WINDOW (dialog),
-                                                  "gnome-dev-mouse-optical");
+                                                  "input-mouse");
                         gtk_dialog_run (GTK_DIALOG (dialog));
                         gtk_widget_destroy (dialog);
                 }
